@@ -1,6 +1,6 @@
 use webcore::value::Reference;
 use webcore::try_from::TryInto;
-use webapi::dom_exception::InvalidCharacterError;
+use webapi::dom_exception::{InvalidCharacterError, InvalidPointerId};
 use webapi::event_target::{IEventTarget, EventTarget};
 use webapi::node::{INode, Node};
 use webapi::token_list::TokenList;
@@ -60,6 +60,46 @@ pub trait IElement: INode + IParentNode {
         ).unwrap()
     }
 
+    /// Gets the the number of pixels that an element's content is scrolled vertically.
+    ///
+    /// [(Javascript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollTop)
+    // https://drafts.csswg.org/cssom-view/#ref-for-dom-element-scrolltop%E2%91%A0
+    fn scroll_top( &self ) -> f64 {
+        js!(
+            return @{self.as_ref()}.scrollTop;
+        ).try_into().unwrap()
+    }
+
+    /// Sets the the number of pixels that an element's content is scrolled vertically.
+    ///
+    /// [(Javascript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollTop)
+    // https://drafts.csswg.org/cssom-view/#ref-for-dom-element-scrolltop%E2%91%A0
+    fn set_scroll_top( &self, value: f64 ) {
+        js! { @(no_return)
+            @{self.as_ref()}.scrollTop = @{value};
+        }
+    }
+
+    /// Gets the the number of pixels that an element's content is scrolled to the left.
+    ///
+    /// [(Javascript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollLeft)
+    // https://drafts.csswg.org/cssom-view/#ref-for-dom-element-scrollleft%E2%91%A0
+    fn scroll_left( &self ) -> f64 {
+        js!(
+            return @{self.as_ref()}.scrollLeft;
+        ).try_into().unwrap()
+    }
+
+    /// Sets the the number of pixels that an element's content is scrolled to the left.
+    ///
+    /// [(Javascript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/scrollLeft)
+    // https://drafts.csswg.org/cssom-view/#ref-for-dom-element-scrollleft%E2%91%A0
+    fn set_scroll_left( &self, value: f64 ) {
+        js! { @(no_return)
+            @{self.as_ref()}.scrollLeft = @{value};
+        }
+    }
+
     /// Element.getAttributeNames() returns the attribute names of the element
     /// as an Array of strings. If the element has no attributes it returns an empty array.
     ///
@@ -91,6 +131,37 @@ pub trait IElement: INode + IParentNode {
             return @{self.as_ref()}.hasAttributes();
         ).try_into().unwrap()
     }
+
+    /// Designates a specific element as the capture target of future pointer events.
+    ///
+    /// [(JavaScript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/setPointerCapture)
+    // https://w3c.github.io/pointerevents/#dom-element-setpointercapture
+    #[inline]
+    fn set_pointer_capture( &self, pointer_id: i32 ) -> Result< (), InvalidPointerId > {
+        js_try!(
+            return @{self.as_ref()}.setPointerCapture( @{pointer_id} );
+        ).unwrap()
+    }
+
+    /// Releases pointer capture that was previously set for a specific pointer
+    ///
+    /// [(JavaScript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/releasePointerCapture)
+    // https://w3c.github.io/pointerevents/#dom-element-releasepointercapture
+    #[inline]
+    fn release_pointer_capture( &self, pointer_id: i32 ) -> Result< (), InvalidPointerId > {
+        js_try!(
+            return @{self.as_ref()}.releasePointerCapture( @{pointer_id} );
+        ).unwrap()
+    }
+
+    /// Returns a boolean indicating if the element has captured the specified pointer
+    ///
+    /// [(JavaScript docs)](https://developer.mozilla.org/en-US/docs/Web/API/Element/hasPointerCapture)
+    // https://w3c.github.io/pointerevents/#dom-element-haspointercapture
+    #[inline]
+    fn has_pointer_capture( &self, pointer_id: i32 ) -> bool {
+        js!( return @{self.as_ref()}.hasPointerCapture( @{pointer_id} ); ).try_into().unwrap()
+    }
 }
 
 /// A reference to a JavaScript object which implements the [IElement](trait.IElement.html)
